@@ -15,10 +15,10 @@
 # OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
 """
-Implement client to access to CouchDB. It allow you to manage a CouchDB server, 
-databases, doc managements and view access. All objects mostly reflect python 
-objects for convenience. Server and Databases objects could be used for example 
-as easy as using a dict.
+Client implementation for CouchDB access. It allows you to manage a CouchDB
+server, databases, documents and views. All objects mostly reflect python 
+objects for convenience. Server and Database objects for example, can be 
+used as easy as a dict. 
 
 Example:
     
@@ -46,7 +46,7 @@ from itertools import groupby
 from mimetypes import guess_type
 import re
 
-from restclient.rest import url_quote
+from restkit.rest import url_quote
 
 from couchdbkit.exceptions import *
 from couchdbkit.resource import CouchdbResource, ResourceNotFound, ResourceConflict
@@ -55,8 +55,8 @@ from couchdbkit.utils import validate_dbname
 DEFAULT_UUID_BATCH_COUNT = 1000
 
 class Server(object):
-    """ Server object that allow you to access and manage a couchdb node. 
-    A Server object could be use like any `dict` object.
+    """ Server object that allows you to access and manage a couchdb node. 
+    A Server object can be used like any `dict` object.
     """
     
     def __init__(self, uri='http://127.0.0.1:5984', uuid_batch_count=DEFAULT_UUID_BATCH_COUNT, 
@@ -65,7 +65,7 @@ class Server(object):
         
         @param uri: uri of CouchDb host
         @param uuid_batch_count: max of uuids to get in one time
-        @param transport: an transport instance from :mod:`restclient.transport`. Could be used
+        @param transport: an transport instance from :mod:`restkit.transport`. Can be used
                 to manage authentification to your server or proxy.
         """
         
@@ -106,8 +106,8 @@ class Server(object):
 
     def get_or_create_db(self, dbname):
         """
-        It try to return a Database object for dbname. If 
-        database don't exit, it will be created.
+        Try to return a Database object for dbname. If 
+        database doest't exist, it will be created.
         
         """
         try:
@@ -134,6 +134,20 @@ class Server(object):
         if not self.uuids:
             self.uuids = self.res.get('/_uuids', count=self._uuid_batch_count)["uuids"]
         return self.uuids.pop()
+        
+    def add_authorization(self, obj_auth):
+        """
+        Allow you to add basic authentification or any authentification 
+        object inherited from `restkit.httpc.Auth`.
+        
+        ex:
+        
+            >>> from couchdbkit import Server
+            >>> from restkit.httpc import BasicAuth
+            >>> server = Server()
+            >>> server.add_authorization(BasicAuth((username, password)))
+        """
+        self.res.add_authorization(obj_auth)
           
     def __getitem__(self, dbname):
         if dbname in self:
@@ -160,7 +174,7 @@ class Server(object):
         
 class Database(object):
     """ Object that abstract access to a CouchDB database
-    A Database object could act as a Dict object.
+    A Database object can act as a Dict object.
     """
 
     def __init__(self, server, dbname):
@@ -195,7 +209,7 @@ class Database(object):
         
     def info(self):
         """
-        Get infos of database
+        Get database information
             
         @return: dict
         """
@@ -214,7 +228,7 @@ class Database(object):
         self.bulk_save( _design_docs )
         
     def doc_exist(self, docid):
-        """Test if document exist in database
+        """Test if document exists in a database
 
         @param docid: str, document id
         @return: boolean, True if document exist
@@ -231,9 +245,9 @@ class Database(object):
         
         Args:
         @param docid: str, document id to retrieve 
-        @param rev: if specified, allow you to retrieve
-        a specifiec revision of document
-        @param wrapper: callable. function that take a dict as param. 
+        @param rev: if specified, allows you to retrieve
+        a specific revision of document
+        @param wrapper: callable. function that takes dict as a param. 
         Used to wrap an object.
         
         @return: dict, representation of CouchDB document as
@@ -253,9 +267,9 @@ class Database(object):
         return doc
 
     def all_docs(self, by_seq=False, **params):
-        """Get all documents from database
+        """Get all documents from a database
 
-        This method has the same behavior than view.
+        This method has the same behavior as a view.
 
         `all_docs( **params )` is the same as `view('_all_docs', **params)`
          and `all_docs( by_seq=True, **params)` is the same as
@@ -265,7 +279,7 @@ class Database(object):
 
         Args:
         @param by_seq: bool, if True the "_all_docs_by_seq" is passed to
-        couchdb. It will return all updated list.
+        couchdb. It will return an updated list of all documents.
 
         @return: list, results of the view
         """
@@ -386,13 +400,13 @@ class Database(object):
     
     def bulk_delete(self, docs, all_or_nothing=False):
         """ bulk delete. 
-        It add '_deleted' member to doc then use bulk_save to save them."""
+        It adds '_deleted' member to doc then uses bulk_save to save them."""
         for doc in docs:
             doc['_deleted'] = True
         self.bulk_save(docs, use_uuids=False, all_or_nothing=all_or_nothing)
  
     def delete_doc(self, doc):
-        """ delete a document or a list of document
+        """ delete a document or a list of documents
         @param doc: str or dict,  document id or full doc.
         @return: dict like:
        
@@ -496,9 +510,9 @@ class Database(object):
         return View(self, '_all_docs', wrapper=wrapper, **params)
     iter_documents = documents    
 
-    def put_attachment(self, doc, content, name=None, 
-            content_type=None, content_length=None, chunked=True):
-        """ Add attachement to a document.
+    def put_attachment(self, doc, content, name=None, content_type=None, 
+        content_length=None):
+        """ Add attachement to a document. All attachments are streamed.
 
         @param doc: dict, document object
         @param content: string or :obj:`File` object.
@@ -531,59 +545,60 @@ class Database(object):
 
         headers = {}
         
-        content = content or ''
-        if content and chunked:
-            headers.setdefault("Transfer-Encoding", "chunked")
-
+        if not content:
+            content = ""
+            content_length = 0
         if name is None:
             if hasattr(content, "name"):
                 name = content.name
             else:
                 raise InvalidAttachment('You should provid a valid attachment name')
-
+        name = url_quote(name, safe="")
         if content_type is None:
             content_type = ';'.join(filter(None, guess_type(name)))
 
         if content_type:
             headers['Content-Type'] = content_type
-
+            
+        # add appropriate headers    
         if content_length and content_length is not None:
             headers['Content-Length'] = content_length
 
         if hasattr(doc, 'to_json'):
-            doc_ = doc.to_json()
+            doc1 = doc.to_json()
         else:
-            doc_ = doc
+            doc1 = doc
 
-        docid = self.escape_docid(doc_['_id'])
-        name = url_quote(name, safe="")
-            
+        docid = self.escape_docid(doc1['_id'])
         res = self.res(docid).put(name, payload=content, 
-                headers=headers, rev=doc_['_rev'])
+                headers=headers, rev=doc1['_rev'])
 
         if res['ok']:
-            doc_.update({ '_rev': res['rev']})
+            doc1.update({ '_rev': res['rev']})
         return res['ok']
 
     def delete_attachment(self, doc, name):
-        """ delete attachement of documen
+        """ delete attachement to the document
 
         @param doc: dict, document object in python
         @param name: name of attachement
     
-        @return: dict, withm member ok setto True if delete was ok.
+        @return: dict, with member ok set to True if delete was ok.
         """
         docid = self.escape_docid(doc['_id'])
         name = url_quote(name, safe="")
         
         return self.res(docid).delete(name, rev=doc['_rev'])
 
-    def fetch_attachment(self, id_or_doc, name):
-        """ get attachment in document
+    def fetch_attachment(self, id_or_doc, name, stream=False, 
+            stream_size=16384):
+        """ get attachment in a document
         
         @param id_or_doc: str or dict, doc id or document dict
         @param name: name of attachment default: default result
-
+        @param stream: boolean, response return a ResponseStream object
+        @param stream_size: int, size in bytes of response stream block
+        
         @return: str, attachment
         """
 
@@ -595,7 +610,8 @@ class Database(object):
         docid = self.escape_docid(docid)
         name = url_quote(name, safe="")
         try:
-            data = self.res(docid).get(name)
+            data = self.res(docid).get(name, _stream=stream, 
+                _stream_size=stream_size)
         except ResourceNotFound:
             return None
         return data
@@ -663,6 +679,7 @@ class ViewResults(object):
         self._result_cache = None
         self._total_rows = None
         self._offset = 0
+        self._dynamic_keys = []
 
     def iterator(self):
         self._fetch_if_needed()
@@ -718,9 +735,25 @@ class ViewResults(object):
 
     def fetch(self):
         """ fetch results and cache them """
+        # reset dynamic keys
+        for key in  self._dynamic_keys:
+            try:
+                delattr(self, key)
+            except:
+                pass
+        self._dynamic_keys = []
+        
         self._result_cache = self.view._exec(**self.params)
         self._total_rows = self._result_cache.get('total_rows')
         self._offset = self._result_cache.get('offset', 0)
+        
+        # add key in view results that could be added by an external
+        # like couchdb-lucene
+        for key in self._result_cache.keys():
+            if key not in ["total_rows", "offset", "rows"]:
+                self._dynamic_keys.append(key)
+                setattr(self, key, self._result_cache[keys])
+        
 
     def _fetch_if_needed(self):
         if not self._result_cache:
