@@ -118,7 +118,7 @@ def document_to_dict(instance, properties=None, exclude=None):
             continue
         if exclude and prop_name in exclude:
             continue
-        data[prop_name] = value_to_python(instance._doc[prop_name])
+        data[prop_name] = instance[prop_name]
     return data
 
 def fields_for_document(document, properties=None, exclude=None):
@@ -133,7 +133,10 @@ def fields_for_document(document, properties=None, exclude=None):
     in the ``properties`` argument.
     """
     field_list = []
-    for prop in document._properties.values(): 
+    values = document._properties.values()
+    values.sort(lambda a, b: cmp(a.creation_counter, b.creation_counter))
+    
+    for prop in values: 
         if properties and not prop.name in properties:
             continue
         if exclude and prop.name in exclude:
@@ -146,7 +149,7 @@ def fields_for_document(document, properties=None, exclude=None):
             }
             
             if prop.default is not None:
-                defaults['initial'] = prop.default_value()
+                defaults['initial'] = prop.default_value
                 
             if prop.choices:
                 if prop.default:
